+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
0063 0073 ; [.1C7B.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER S>
0063 0053 ; [.1C7B.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER C, LATIN CAPITAL LETTER S>
0043 0073 ; [.1C7B.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER S>
0043 0053 ; [.1C7B.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER S>
0064 007A ; [.1C90.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z>
0064 005A ; [.1C90.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z>
0044 007A ; [.1C90.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z>
0044 005A ; [.1C90.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z>
0064 007A 0073 ; [.1C91.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0064 007A 0053 ; [.1C91.0020.0007][.0000.0000.0002][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0064 005A 0073 ; [.1C91.0020.0007][.0000.0000.0002][.0000.0000.0007] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0064 005A 0053 ; [.1C91.0020.0007][.0000.0000.0002][.0000.0000.0008] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0044 007A 0073 ; [.1C91.0020.0007][.0000.0000.0008][.0000.0000.0002] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0044 007A 0053 ; [.1C91.0020.0007][.0000.0000.0008][.0000.0000.0007] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0044 005A 0073 ; [.1C91.0020.0007][.0000.0000.0008][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0044 005A 0053 ; [.1C91.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0067 0079 ; [.1CF5.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER Y>
0067 0059 ; [.1CF5.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER Y>
0047 0079 ; [.1CF5.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER Y>
0047 0059 ; [.1CF5.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER Y>
006C 0079 ; [.1D78.0020.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER Y>
006C 0059 ; [.1D78.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER L, LATIN CAPITAL LETTER Y>
004C 0079 ; [.1D78.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER Y>
004C 0059 ; [.1D78.0020.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER Y>
006E 0079 ; [.1DBA.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
006E 0059 ; [.1DBA.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER N, LATIN CAPITAL LETTER Y>
004E 0079 ; [.1DBA.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 0059 ; [.1DBA.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
0073 007A ; [.1E72.0020.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER Z>
0073 005A ; [.1E72.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER S, LATIN CAPITAL LETTER Z>
0053 007A ; [.1E72.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER Z>
0053 005A ; [.1E72.0020.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER Z>
0074 0079 ; [.1E96.0020.0002] # <LATIN SMALL LETTER T, LATIN SMALL LETTER Y>
0074 0059 ; [.1E96.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER T, LATIN CAPITAL LETTER Y>
0054 0079 ; [.1E96.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER Y>
0054 0059 ; [.1E96.0020.0008] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER Y>
007A 0073 ; [.1F22.0020.0002] # <LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
007A 0053 ; [.1F22.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
005A 0073 ; [.1F22.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
005A 0053 ; [.1F22.0020.0008] # <LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
00F6      ; [.1DDE.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.1DDE.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.1DDE.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.1DDE.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.1DDE.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.1DDE.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
0151      ; [.1DDE.0021.0002] # LATIN SMALL LETTER O WITH DOUBLE ACUTE
006F 030B ; [.1DDE.0021.0002] # LATIN SMALL LETTER O WITH DOUBLE ACUTE
0150      ; [.1DDE.0021.0008] # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
004F 030B ; [.1DDE.0021.0008] # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
00FC      ; [.1EB6.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
0075 0308 ; [.1EB6.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
00DC      ; [.1EB6.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
0055 0308 ; [.1EB6.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
01DC      ; [.1EB6.0020.0002][.0000.0025.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
01DB      ; [.1EB6.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
01D8      ; [.1EB6.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
01D7      ; [.1EB6.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
01D6      ; [.1EB6.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
01D5      ; [.1EB6.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
01DA      ; [.1EB6.0020.0002][.0000.0028.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
01D9      ; [.1EB6.0020.0008][.0000.0028.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
0171      ; [.1EB6.0021.0002] # LATIN SMALL LETTER U WITH DOUBLE ACUTE
0075 030B ; [.1EB6.0021.0002] # LATIN SMALL LETTER U WITH DOUBLE ACUTE
0170      ; [.1EB6.0021.0008] # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
0055 030B ; [.1EB6.0021.0008] # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
0063 0063 0073 ; [.1C7B.0020.0002][.1C7B.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER C, LATIN SMALL LETTER S>
0063 0063 0053 ; [.1C7B.0020.0002][.1C7B.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER C, LATIN CAPITAL LETTER S>
0063 0043 0073 ; [.1C7B.0020.0002][.1C7B.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER C, LATIN CAPITAL LETTER C, LATIN SMALL LETTER S>
0063 0043 0053 ; [.1C7B.0020.0002][.1C7B.0020.0008] # <LATIN SMALL LETTER C, LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER S>
0043 0063 0073 ; [.1C7B.0020.0008][.1C7B.0020.0002] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER C, LATIN SMALL LETTER S>
0043 0063 0053 ; [.1C7B.0020.0008][.1C7B.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER C, LATIN CAPITAL LETTER S>
0043 0043 0073 ; [.1C7B.0020.0008][.1C7B.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER C, LATIN SMALL LETTER S>
0043 0043 0053 ; [.1C7B.0020.0008][.1C7B.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER S>
0064 0064 007A ; [.1C90.0020.0002][.1C90.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z>
0064 0064 005A ; [.1C90.0020.0002][.1C90.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z>
0064 0044 007A ; [.1C90.0020.0002][.1C90.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z>
0064 0044 005A ; [.1C90.0020.0002][.1C90.0020.0008] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z>
0044 0064 007A ; [.1C90.0020.0008][.1C90.0020.0002] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z>
0044 0064 005A ; [.1C90.0020.0008][.1C90.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z>
0044 0044 007A ; [.1C90.0020.0008][.1C90.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z>
0044 0044 005A ; [.1C90.0020.0008][.1C90.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z>
0064 0064 007A 0073 ; [.1C91.0020.0002][.1C91.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0064 0064 007A 0053 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0064 0064 005A 0073 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0007] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0064 0064 005A 0053 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0008] # <LATIN SMALL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0064 0044 007A 0073 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0002] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0064 0044 007A 0053 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0007] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0064 0044 005A 0073 ; [.1C91.0020.0002][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0008] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0064 0044 005A 0053 ; [.1C91.0020.0002][.1C91.0020.0008] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0044 0064 007A 0073 ; [.1C91.0020.0008][.1C91.0020.0002] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0044 0064 007A 0053 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0002] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0044 0064 005A 0073 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0007] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0044 0064 005A 0053 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0002][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0044 0044 007A 0073 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0002] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
0044 0044 007A 0053 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0007] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
0044 0044 005A 0073 ; [.1C91.0020.0008][.1C91.0020.0007][.0000.0000.0008][.0000.0000.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
0044 0044 005A 0053 ; [.1C91.0020.0008][.1C91.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
0067 0067 0079 ; [.1CF5.0020.0002][.1CF5.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER G, LATIN SMALL LETTER Y>
0067 0067 0059 ; [.1CF5.0020.0002][.1CF5.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER G, LATIN CAPITAL LETTER Y>
0067 0047 0079 ; [.1CF5.0020.0002][.1CF5.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER G, LATIN SMALL LETTER Y>
0067 0047 0059 ; [.1CF5.0020.0002][.1CF5.0020.0008] # <LATIN SMALL LETTER G, LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER Y>
0047 0067 0079 ; [.1CF5.0020.0008][.1CF5.0020.0002] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER G, LATIN SMALL LETTER Y>
0047 0067 0059 ; [.1CF5.0020.0008][.1CF5.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER G, LATIN CAPITAL LETTER Y>
0047 0047 0079 ; [.1CF5.0020.0008][.1CF5.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER G, LATIN SMALL LETTER Y>
0047 0047 0059 ; [.1CF5.0020.0008][.1CF5.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER Y>
006C 006C 0079 ; [.1D78.0020.0002][.1D78.0020.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER L, LATIN SMALL LETTER Y>
006C 006C 0059 ; [.1D78.0020.0002][.1D78.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER L, LATIN CAPITAL LETTER Y>
006C 004C 0079 ; [.1D78.0020.0002][.1D78.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER L, LATIN CAPITAL LETTER L, LATIN SMALL LETTER Y>
006C 004C 0059 ; [.1D78.0020.0002][.1D78.0020.0008] # <LATIN SMALL LETTER L, LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER Y>
004C 006C 0079 ; [.1D78.0020.0008][.1D78.0020.0002] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER L, LATIN SMALL LETTER Y>
004C 006C 0059 ; [.1D78.0020.0008][.1D78.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER L, LATIN CAPITAL LETTER Y>
004C 004C 0079 ; [.1D78.0020.0008][.1D78.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER L, LATIN SMALL LETTER Y>
004C 004C 0059 ; [.1D78.0020.0008][.1D78.0020.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER Y>
006E 006E 0079 ; [.1DBA.0020.0002][.1DBA.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
006E 006E 0059 ; [.1DBA.0020.0002][.1DBA.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER N, LATIN CAPITAL LETTER Y>
006E 004E 0079 ; [.1DBA.0020.0002][.1DBA.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER N, LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
006E 004E 0059 ; [.1DBA.0020.0002][.1DBA.0020.0008] # <LATIN SMALL LETTER N, LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
004E 006E 0079 ; [.1DBA.0020.0008][.1DBA.0020.0002] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
004E 006E 0059 ; [.1DBA.0020.0008][.1DBA.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER N, LATIN CAPITAL LETTER Y>
004E 004E 0079 ; [.1DBA.0020.0008][.1DBA.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 004E 0059 ; [.1DBA.0020.0008][.1DBA.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
0073 0073 007A ; [.1E72.0020.0002][.1E72.0020.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER S, LATIN SMALL LETTER Z>
0073 0073 005A ; [.1E72.0020.0002][.1E72.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER S, LATIN SMALL LETTER S, LATIN CAPITAL LETTER Z>
0073 0053 007A ; [.1E72.0020.0002][.1E72.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER S, LATIN CAPITAL LETTER S, LATIN SMALL LETTER Z>
0073 0053 005A ; [.1E72.0020.0002][.1E72.0020.0008] # <LATIN SMALL LETTER S, LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER Z>
0053 0073 007A ; [.1E72.0020.0008][.1E72.0020.0002] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER S, LATIN SMALL LETTER Z>
0053 0073 005A ; [.1E72.0020.0008][.1E72.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER S, LATIN SMALL LETTER S, LATIN CAPITAL LETTER Z>
0053 0053 007A ; [.1E72.0020.0008][.1E72.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER S, LATIN SMALL LETTER Z>
0053 0053 005A ; [.1E72.0020.0008][.1E72.0020.0008] # <LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER S, LATIN CAPITAL LETTER Z>
0074 0074 0079 ; [.1E96.0020.0002][.1E96.0020.0002] # <LATIN SMALL LETTER T, LATIN SMALL LETTER T, LATIN SMALL LETTER Y>
0074 0074 0059 ; [.1E96.0020.0002][.1E96.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER T, LATIN SMALL LETTER T, LATIN CAPITAL LETTER Y>
0074 0054 0079 ; [.1E96.0020.0002][.1E96.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER T, LATIN CAPITAL LETTER T, LATIN SMALL LETTER Y>
0074 0054 0059 ; [.1E96.0020.0002][.1E96.0020.0008] # <LATIN SMALL LETTER T, LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER Y>
0054 0074 0079 ; [.1E96.0020.0008][.1E96.0020.0002] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER T, LATIN SMALL LETTER Y>
0054 0074 0059 ; [.1E96.0020.0008][.1E96.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER T, LATIN CAPITAL LETTER Y>
0054 0054 0079 ; [.1E96.0020.0008][.1E96.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER T, LATIN SMALL LETTER Y>
0054 0054 0059 ; [.1E96.0020.0008][.1E96.0020.0008] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER Y>
007A 007A 0073 ; [.1F22.0020.0002][.1F22.0020.0002] # <LATIN SMALL LETTER Z, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
007A 007A 0053 ; [.1F22.0020.0002][.1F22.0020.0007][.0000.0000.0002] # <LATIN SMALL LETTER Z, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
007A 005A 0073 ; [.1F22.0020.0002][.1F22.0020.0007][.0000.0000.0008] # <LATIN SMALL LETTER Z, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
007A 005A 0053 ; [.1F22.0020.0002][.1F22.0020.0008] # <LATIN SMALL LETTER Z, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
005A 007A 0073 ; [.1F22.0020.0008][.1F22.0020.0002] # <LATIN CAPITAL LETTER Z, LATIN SMALL LETTER Z, LATIN SMALL LETTER S>
005A 007A 0053 ; [.1F22.0020.0008][.1F22.0020.0007][.0000.0000.0002] # <LATIN CAPITAL LETTER Z, LATIN SMALL LETTER Z, LATIN CAPITAL LETTER S>
005A 005A 0073 ; [.1F22.0020.0008][.1F22.0020.0007][.0000.0000.0008] # <LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER Z, LATIN SMALL LETTER S>
005A 005A 0053 ; [.1F22.0020.0008][.1F22.0020.0008] # <LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER Z, LATIN CAPITAL LETTER S>
ENTRY
};
