+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
1EB9      ; [.1CAB.0020.0002] # LATIN SMALL LETTER E WITH DOT BELOW
0065 0323 ; [.1CAB.0020.0002] # LATIN SMALL LETTER E WITH DOT BELOW
1EB8      ; [.1CAB.0020.0008] # LATIN CAPITAL LETTER E WITH DOT BELOW
0045 0323 ; [.1CAB.0020.0008] # LATIN CAPITAL LETTER E WITH DOT BELOW
1EC7      ; [.1CAB.0020.0002][.0000.0027.0002] # LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW
1EC6      ; [.1CAB.0020.0008][.0000.0027.0002] # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW
0067 0062 ; [.1CF5.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER B>
0047 0062 ; [.1CF5.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER B>
0047 0042 ; [.1CF5.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER B>
1ECD      ; [.1DDE.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
006F 0323 ; [.1DDE.0020.0002] # LATIN SMALL LETTER O WITH DOT BELOW
1ECC      ; [.1DDE.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
004F 0323 ; [.1DDE.0020.0008] # LATIN CAPITAL LETTER O WITH DOT BELOW
1ED9      ; [.1DDE.0020.0002][.0000.0027.0002] # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1ED8      ; [.1DDE.0020.0008][.0000.0027.0002] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
1EE3      ; [.1DDE.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER O WITH HORN AND DOT BELOW
1EE2      ; [.1DDE.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW
1E63      ; [.1E72.0020.0002] # LATIN SMALL LETTER S WITH DOT BELOW
0073 0323 ; [.1E72.0020.0002] # LATIN SMALL LETTER S WITH DOT BELOW
1E62      ; [.1E72.0020.0008] # LATIN CAPITAL LETTER S WITH DOT BELOW
0053 0323 ; [.1E72.0020.0008] # LATIN CAPITAL LETTER S WITH DOT BELOW
ENTRY
};
