+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
0D3D      ; [.0000.00C6.0002] # MALAYALAM SIGN AVAGRAHA
0D57      ; [.28CD.0020.0002] # MALAYALAM AU LENGTH MARK
0D4C      ; [.28CD.0020.0003] # MALAYALAM VOWEL SIGN AU
0D46 0D57 ; [.28CD.0020.0003] # MALAYALAM VOWEL SIGN AU
0D15 0D4D 200D ; [.2898.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER KA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7F      ; [.2898.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU K
0D23 0D4D 200D ; [.28A6.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER NNA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7A      ; [.28A6.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU NN
0D28 0D4D 200D ; [.28AB.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER NA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7B      ; [.28AB.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU N
0D30 0D4D 200D ; [.28B3.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER RA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7C      ; [.28B3.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU RR
0D32 0D4D 200D ; [.28B4.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER LA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7D      ; [.28B4.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU L
0D33 0D4D 200D ; [.28BA.0020.0002][.28CF.0021.0002] # <MALAYALAM LETTER LLA, MALAYALAM SIGN VIRAMA, ZERO WIDTH JOINER>
0D7E      ; [.28BA.0020.0003][.28CF.0021.0002] # MALAYALAM LETTER CHILLU LL
0D02      ; [.28B1.0020.0002][.28CF.0021.0002] # MALAYALAM SIGN ANUSVARA
0D7B 0D4D ; [.28AB.0020.0002][.28CF.0020.0003] # <MALAYALAM LETTER CHILLU N, MALAYALAM SIGN VIRAMA>
ENTRY
};
