+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
00F1      ; [.1DBA.0020.0002] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.1DBA.0020.0002] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.1DBA.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.1DBA.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
ENTRY
};
