+{
   locale_version => 1.25,
# d-tail, open-e, f-hook, gamma, eng, open-o, v-hook don't require tailoring
   entry => <<'ENTRY', # for DUCET v9.0.0
0302      ; [.0000.0029.0002] # COMBINING CIRCUMFLEX ACCENT
00E2      ; [.1C47.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER A WITH CIRCUMFLEX
00C2      ; [.1C47.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
00EA      ; [.1CAA.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER E WITH CIRCUMFLEX
00CA      ; [.1CAA.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
00EE      ; [.1D32.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER I WITH CIRCUMFLEX
00CE      ; [.1D32.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
00F4      ; [.1DDD.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER O WITH CIRCUMFLEX
00D4      ; [.1DDD.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
00FB      ; [.1EB5.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER U WITH CIRCUMFLEX
00DB      ; [.1EB5.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
0177      ; [.1F0B.0020.0002][.0000.0029.0002] # LATIN SMALL LETTER Y WITH CIRCUMFLEX
0176      ; [.1F0B.0020.0008][.0000.0029.0002] # LATIN CAPITAL LETTER Y WITH CIRCUMFLEX
0064 007A ; [.1C90.0020.0002] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z>
0044 007A ; [.1C90.0020.0007] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z>
0044 005A ; [.1C90.0020.0008] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z>
0067 0062 ; [.1CF5.0020.0002] # <LATIN SMALL LETTER G, LATIN SMALL LETTER B>
0047 0062 ; [.1CF5.0020.0007] # <LATIN CAPITAL LETTER G, LATIN SMALL LETTER B>
0047 0042 ; [.1CF5.0020.0008] # <LATIN CAPITAL LETTER G, LATIN CAPITAL LETTER B>
0078      ; [.1D19.0020.0002] # LATIN SMALL LETTER X
0058      ; [.1D19.0020.0008] # LATIN CAPITAL LETTER X
006B 0070 ; [.1D66.0020.0002] # <LATIN SMALL LETTER K, LATIN SMALL LETTER P>
004B 0070 ; [.1D66.0020.0007] # <LATIN CAPITAL LETTER K, LATIN SMALL LETTER P>
004B 0050 ; [.1D66.0020.0008] # <LATIN CAPITAL LETTER K, LATIN CAPITAL LETTER P>
006E 0079 ; [.1DBA.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER Y>
004E 0079 ; [.1DBA.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER Y>
004E 0059 ; [.1DBA.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER Y>
0074 0073 ; [.1E96.0020.0002] # <LATIN SMALL LETTER T, LATIN SMALL LETTER S>
0054 0073 ; [.1E96.0020.0007] # <LATIN CAPITAL LETTER T, LATIN SMALL LETTER S>
0054 0053 ; [.1E96.0020.0008] # <LATIN CAPITAL LETTER T, LATIN CAPITAL LETTER S>
ENTRY
};
