+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
04D9      ; [.205B.0020.0002] # CYRILLIC SMALL LETTER SCHWA
04D8      ; [.205B.0020.0008] # CYRILLIC CAPITAL LETTER SCHWA
ENTRY
};
