+{
   locale_version => 1.25,
   entry => <<'ENTRY', # for DUCET v9.0.0
00E1      ; [.1C5F.0020.0002] # LATIN SMALL LETTER A WITH ACUTE
0061 0301 ; [.1C5F.0020.0002] # LATIN SMALL LETTER A WITH ACUTE
0061 0341 ; [.1C5F.0020.0002] # LATIN SMALL LETTER A WITH ACUTE
00C1      ; [.1C5F.0020.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0301 ; [.1C5F.0020.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0041 0341 ; [.1C5F.0020.0008] # LATIN CAPITAL LETTER A WITH ACUTE
0111      ; [.1C8F.0021.0002] # LATIN SMALL LETTER D WITH STROKE
0064 0335 ; [.1C8F.0021.0002] # LATIN SMALL LETTER D WITH STROKE
0110      ; [.1C8F.0021.0008] # LATIN CAPITAL LETTER D WITH STROKE
0044 0335 ; [.1C8F.0021.0008] # LATIN CAPITAL LETTER D WITH STROKE
00F0      ; [.1C90.0020.0002] # LATIN SMALL LETTER ETH
1DD9      ; [.1C90.0020.0002] # COMBINING LATIN SMALL LETTER ETH
00D0      ; [.1C90.0020.0008] # LATIN CAPITAL LETTER ETH
00E9      ; [.1CE4.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0301 ; [.1CE4.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
0065 0341 ; [.1CE4.0020.0002] # LATIN SMALL LETTER E WITH ACUTE
00C9      ; [.1CE4.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0301 ; [.1CE4.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
0045 0341 ; [.1CE4.0020.0008] # LATIN CAPITAL LETTER E WITH ACUTE
00ED      ; [.1D4B.0020.0002] # LATIN SMALL LETTER I WITH ACUTE
0069 0301 ; [.1D4B.0020.0002] # LATIN SMALL LETTER I WITH ACUTE
0069 0341 ; [.1D4B.0020.0002] # LATIN SMALL LETTER I WITH ACUTE
00CD      ; [.1D4B.0020.0008] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0301 ; [.1D4B.0020.0008] # LATIN CAPITAL LETTER I WITH ACUTE
0049 0341 ; [.1D4B.0020.0008] # LATIN CAPITAL LETTER I WITH ACUTE
00F3      ; [.1E0B.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0301 ; [.1E0B.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
006F 0341 ; [.1E0B.0020.0002] # LATIN SMALL LETTER O WITH ACUTE
00D3      ; [.1E0B.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0301 ; [.1E0B.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
004F 0341 ; [.1E0B.0020.0008] # LATIN CAPITAL LETTER O WITH ACUTE
00FA      ; [.1EE2.0020.0002] # LATIN SMALL LETTER U WITH ACUTE
0075 0301 ; [.1EE2.0020.0002] # LATIN SMALL LETTER U WITH ACUTE
0075 0341 ; [.1EE2.0020.0002] # LATIN SMALL LETTER U WITH ACUTE
00DA      ; [.1EE2.0020.0008] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0301 ; [.1EE2.0020.0008] # LATIN CAPITAL LETTER U WITH ACUTE
0055 0341 ; [.1EE2.0020.0008] # LATIN CAPITAL LETTER U WITH ACUTE
00FD      ; [.1F20.0020.0002] # LATIN SMALL LETTER Y WITH ACUTE
0079 0301 ; [.1F20.0020.0002] # LATIN SMALL LETTER Y WITH ACUTE
0079 0341 ; [.1F20.0020.0002] # LATIN SMALL LETTER Y WITH ACUTE
00DD      ; [.1F20.0020.0008] # LATIN CAPITAL LETTER Y WITH ACUTE
0059 0301 ; [.1F20.0020.0008] # LATIN CAPITAL LETTER Y WITH ACUTE
0059 0341 ; [.1F20.0020.0008] # LATIN CAPITAL LETTER Y WITH ACUTE
00E6      ; [.1F96.0020.0002] # LATIN SMALL LETTER AE
1DD4      ; [.1F96.0020.0002] # COMBINING LATIN SMALL LETTER AE
00C6      ; [.1F96.0020.0008] # LATIN CAPITAL LETTER AE
1D2D      ; [.1F96.0020.0014] # MODIFIER LETTER CAPITAL AE
01FD      ; [.1F96.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER AE WITH ACUTE
01FC      ; [.1F96.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER AE WITH ACUTE
01E3      ; [.1F96.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER AE WITH MACRON
01E2      ; [.1F96.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER AE WITH MACRON
00E4      ; [.1F96.0021.0002] # LATIN SMALL LETTER A WITH DIAERESIS
0061 0308 ; [.1F96.0021.0002] # LATIN SMALL LETTER A WITH DIAERESIS
00C4      ; [.1F96.0021.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
0041 0308 ; [.1F96.0021.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
01DF      ; [.1F96.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
01DE      ; [.1F96.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
00F6      ; [.1F97.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.1F97.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.1F97.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.1F97.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.1F97.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.1F97.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
00F8      ; [.1F97.0021.0002] # LATIN SMALL LETTER O WITH STROKE
006F 0338 ; [.1F97.0021.0002] # LATIN SMALL LETTER O WITH STROKE
00D8      ; [.1F97.0021.0008] # LATIN CAPITAL LETTER O WITH STROKE
004F 0338 ; [.1F97.0021.0008] # LATIN CAPITAL LETTER O WITH STROKE
01FF      ; [.1F97.0021.0002][.0000.0024.0002] # LATIN SMALL LETTER O WITH STROKE AND ACUTE
01FE      ; [.1F97.0021.0008][.0000.0024.0002] # LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
00E5      ; [.1F98.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
0061 030A ; [.1F98.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
00C5      ; [.1F98.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
0041 030A ; [.1F98.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
212B      ; [.1F98.0020.0008] # ANGSTROM SIGN
01FB      ; [.1F98.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
01FA      ; [.1F98.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
ENTRY
};
